@@ -1,22 +1,26 @@
+from __future__ import annotations
+
 import uuid
 from collections import OrderedDict
 from functools import lru_cache, partial
 from html import escape
-
-import pkg_resources
+from importlib.resources import read_binary
 
 from .formatting import inline_variable_array_repr, short_data_repr
 from .options import _get_boolean_with_default
 
-STATIC_FILES = ("static/html/icons-svg-inline.html", "static/css/style.css")
+STATIC_FILES = (
+    ("xarray.static.html", "icons-svg-inline.html"),
+    ("xarray.static.css", "style.css"),
+)
 
 
 @lru_cache(None)
 def _load_static_files():
     """Lazily load the resource files into memory the first time they are needed"""
     return [
-        pkg_resources.resource_string("xarray", fname).decode("utf8")
-        for fname in STATIC_FILES
+        read_binary(package, resource).decode("utf-8")
+        for package, resource in STATIC_FILES
     ]
 
 
@@ -29,12 +33,12 @@ def short_data_repr_html(array):
     return f"<pre>{text}</pre>"
 
 
-def format_dims(dims, coord_names):
+def format_dims(dims, dims_with_index):
     if not dims:
         return ""
 
     dim_css_map = {
-        k: " class='xr-has-index'" if k in coord_names else "" for k, v in dims.items()
+        dim: " class='xr-has-index'" if dim in dims_with_index else "" for dim in dims
     }
 
     dims_li = "".join(
@@ -64,38 +68,7 @@ def _icon(icon_name):
     )
 
 
-def _summarize_coord_multiindex(name, coord):
-    preview = f"({', '.join(escape(l) for l in coord.level_names)})"
-    return summarize_variable(
-        name, coord, is_index=True, dtype="MultiIndex", preview=preview
-    )
-
-
-def summarize_coord(name, var):
-    is_index = name in var.dims
-    if is_index:
-        coord = var.variable.to_index_variable()
-        if coord.level_names is not None:
-            coords = {name: _summarize_coord_multiindex(name, coord)}
-            for lname in coord.level_names:
-                var = coord.get_level_variable(lname)
-                coords[lname] = summarize_variable(lname, var)
-            return coords
-
-    return {name: summarize_variable(name, var, is_index)}
-
-
-def summarize_coords(variables):
-    coords = {}
-    for k, v in variables.items():
-        coords.update(**summarize_coord(k, v))
-
-    vars_li = "".join(f"<li class='xr-var-item'>{v}</li>" for v in coords.values())
-
-    return f"<ul class='xr-var-list'>{vars_li}</ul>"
-
-
-def summarize_variable(name, var, is_index=False, dtype=None, preview=None):
+def summarize_variable(name, var, is_index=False, dtype=None):
     variable = var.variable if hasattr(var, "variable") else var
 
     cssclass_idx = " class='xr-has-index'" if is_index else ""
@@ -108,7 +81,7 @@ def summarize_variable(name, var, is_index=False, dtype=None, preview=None):
     data_id = "data-" + str(uuid.uuid4())
     disabled = "" if len(var.attrs) else "disabled"
 
-    preview = preview or escape(inline_variable_array_repr(variable, 35))
+    preview = escape(inline_variable_array_repr(variable, 35))
     attrs_ul = summarize_attrs(var.attrs)
     data_repr = short_data_repr_html(variable)
 
@@ -132,6 +105,17 @@ def summarize_variable(name, var, is_index=False, dtype=None, preview=None):
     )
 
 
+def summarize_coords(variables):
+    li_items = []
+    for k, v in variables.items():
+        li_content = summarize_variable(k, v, is_index=k in variables.xindexes)
+        li_items.append(f"<li class='xr-var-item'>{li_content}</li>")
+
+    vars_li = "".join(li_items)
+
+    return f"<ul class='xr-var-list'>{vars_li}</ul>"
+
+
 def summarize_vars(variables):
     vars_li = "".join(
         f"<li class='xr-var-item'>{summarize_variable(k, v)}</li>"
@@ -182,7 +166,7 @@ def _mapping_section(
 
 
 def dim_section(obj):
-    dim_list = format_dims(obj.dims, list(obj.coords))
+    dim_list = format_dims(obj.dims, obj.xindexes.dims)
 
     return collapsible_section(
         "Dimensions", inline_details=dim_list, enabled=False, collapsed=True
@@ -253,7 +237,7 @@ def _obj_repr(obj, header_components, sections):
         "<div>"
         f"{icons_svg}<style>{css_style}</style>"
         f"<pre class='xr-text-repr-fallback'>{escape(repr(obj))}</pre>"
-        "<div class='xr-wrap' hidden>"
+        "<div class='xr-wrap' style='display:none'>"
         f"{header}"
         f"<ul class='xr-sections'>{sections}</ul>"
         "</div>"
@@ -263,15 +247,18 @@ def _obj_repr(obj, header_components, sections):
 
 def array_repr(arr):
     dims = OrderedDict((k, v) for k, v in zip(arr.dims, arr.shape))
+    if hasattr(arr, "xindexes"):
+        indexed_dims = arr.xindexes.dims
+    else:
+        indexed_dims = {}
 
-    obj_type = "xarray.{}".format(type(arr).__name__)
+    obj_type = f"xarray.{type(arr).__name__}"
     arr_name = f"'{arr.name}'" if getattr(arr, "name", None) else ""
-    coord_names = list(arr.coords) if hasattr(arr, "coords") else []
 
     header_components = [
         f"<div class='xr-obj-type'>{obj_type}</div>",
         f"<div class='xr-array-name'>{arr_name}</div>",
-        format_dims(dims, coord_names),
+        format_dims(dims, indexed_dims),
     ]
 
     sections = [array_section(arr)]
@@ -285,7 +272,7 @@ def array_repr(arr):
 
 
 def dataset_repr(ds):
-    obj_type = "xarray.{}".format(type(ds).__name__)
+    obj_type = f"xarray.{type(ds).__name__}"
 
     header_components = [f"<div class='xr-obj-type'>{escape(obj_type)}</div>"]
 
