@@ -22,6 +22,8 @@ if TYPE_CHECKING:
         "display_expand_coords",
         "display_expand_data_vars",
         "display_expand_data",
+        "display_expand_indexes",
+        "display_default_indexes",
         "enable_cftimeindex",
         "file_cache_maxsize",
         "keep_attrs",
@@ -42,6 +44,8 @@ if TYPE_CHECKING:
         display_expand_coords: Literal["default", True, False]
         display_expand_data_vars: Literal["default", True, False]
         display_expand_data: Literal["default", True, False]
+        display_expand_indexes: Literal["default", True, False]
+        display_default_indexes: Literal["default", True, False]
         enable_cftimeindex: bool
         file_cache_maxsize: int
         keep_attrs: Literal["default", True, False]
@@ -62,6 +66,8 @@ OPTIONS: T_Options = {
     "display_expand_coords": "default",
     "display_expand_data_vars": "default",
     "display_expand_data": "default",
+    "display_expand_indexes": "default",
+    "display_default_indexes": False,
     "enable_cftimeindex": True,
     "file_cache_maxsize": 128,
     "keep_attrs": "default",
@@ -88,6 +94,8 @@ _VALIDATORS = {
     "display_expand_coords": lambda choice: choice in [True, False, "default"],
     "display_expand_data_vars": lambda choice: choice in [True, False, "default"],
     "display_expand_data": lambda choice: choice in [True, False, "default"],
+    "display_expand_indexes": lambda choice: choice in [True, False, "default"],
+    "display_default_indexes": lambda choice: choice in [True, False, "default"],
     "enable_cftimeindex": lambda value: isinstance(value, bool),
     "file_cache_maxsize": _positive_integer,
     "keep_attrs": lambda choice: choice in [True, False, "default"],
