@@ -3,18 +3,38 @@ from __future__ import annotations
 import functools
 import itertools
 import warnings
+from typing import TYPE_CHECKING, Any, Callable, Hashable, Iterable, Literal
 
 import numpy as np
 
 from ..core.formatting import format_item
 from .utils import (
+    _LINEWIDTH_RANGE,
+    _MARKERSIZE_RANGE,
+    _add_legend,
+    _determine_guide,
     _get_nice_quiver_magnitude,
+    _infer_meta_data,
     _infer_xy_labels,
+    _Normalize,
+    _parse_size,
     _process_cmap_cbar_kwargs,
     import_matplotlib_pyplot,
     label_from_attrs,
 )
 
+if TYPE_CHECKING:
+    from matplotlib.axes import Axes
+    from matplotlib.cm import ScalarMappable
+    from matplotlib.colorbar import Colorbar
+    from matplotlib.figure import Figure
+    from matplotlib.legend import Legend
+    from matplotlib.quiver import QuiverKey
+    from matplotlib.text import Annotation
+
+    from ..core.dataarray import DataArray
+    from ..core.types import HueStyleOptions, Self
+
 # Overrides axes.labelsize, xtick.major.size, ytick.major.size
 # from mpl.rcParams
 _FONTSIZE = "small"
@@ -64,9 +84,9 @@ class FacetGrid:
     axes : ndarray of matplotlib.axes.Axes
         Array containing axes in corresponding position, as returned from
         :py:func:`matplotlib.pyplot.subplots`.
-    col_labels : list of matplotlib.text.Text
+    col_labels : list of matplotlib.text.Annotation
         Column titles.
-    row_labels : list of matplotlib.text.Text
+    row_labels : list of matplotlib.text.Annotation
         Row titles.
     fig : matplotlib.figure.Figure
         The figure containing all the axes.
@@ -76,19 +96,42 @@ class FacetGrid:
         sometimes the rightmost grid positions in the bottom row.
     """
 
+    data: DataArray
+    name_dicts: np.ndarray
+    fig: Figure
+    axes: np.ndarray
+    row_names: list[np.ndarray]
+    col_names: list[np.ndarray]
+    figlegend: Legend | None
+    quiverkey: QuiverKey | None
+    cbar: Colorbar | None
+    _single_group: bool | Hashable
+    _nrow: int
+    _row_var: Hashable | None
+    _ncol: int
+    _col_var: Hashable | None
+    _col_wrap: int | None
+    row_labels: list[Annotation | None]
+    col_labels: list[Annotation | None]
+    _x_var: None
+    _y_var: None
+    _cmap_extend: Any | None
+    _mappables: list[ScalarMappable]
+    _finalized: bool
+
     def __init__(
         self,
-        data,
-        col=None,
-        row=None,
-        col_wrap=None,
-        sharex=True,
-        sharey=True,
-        figsize=None,
-        aspect=1,
-        size=3,
-        subplot_kws=None,
-    ):
+        data: DataArray,
+        col: Hashable | None = None,
+        row: Hashable | None = None,
+        col_wrap: int | None = None,
+        sharex: bool = True,
+        sharey: bool = True,
+        figsize: Iterable[float] | None = None,
+        aspect: float = 1,
+        size: float = 3,
+        subplot_kws: dict[str, Any] | None = None,
+    ) -> None:
         """
         Parameters
         ----------
@@ -104,13 +147,13 @@ class FacetGrid:
             If true, the facets will share *x* axes.
         sharey : bool, optional
             If true, the facets will share *y* axes.
-        figsize : tuple, optional
+        figsize : Iterable of float or None, optional
             A tuple (width, height) of the figure in inches.
             If set, overrides ``size`` and ``aspect``.
-        aspect : scalar, optional
+        aspect : scalar, default: 1
             Aspect ratio of each facet, so that ``aspect * size`` gives the
             width of each facet in inches.
-        size : scalar, optional
+        size : scalar, default: 3
             Height (in inches) of each facet. See also: ``aspect``.
         subplot_kws : dict, optional
             Dictionary of keyword arguments for Matplotlib subplots
@@ -130,6 +173,7 @@ class FacetGrid:
             )
 
         # single_group is the grouping variable, if there is exactly one
+        single_group: bool | Hashable
         if col and row:
             single_group = False
             nrow = len(data[row])
@@ -181,14 +225,18 @@ class FacetGrid:
         row_names = list(data[row].to_numpy()) if row else []
 
         if single_group:
-            full = [{single_group: x} for x in data[single_group].to_numpy()]
-            empty = [None for x in range(nrow * ncol - len(full))]
-            name_dicts = full + empty
+            full: list[dict[Hashable, Any] | None] = [
+                {single_group: x} for x in data[single_group].to_numpy()
+            ]
+            empty: list[dict[Hashable, Any] | None] = [
+                None for x in range(nrow * ncol - len(full))
+            ]
+            name_dict_list = full + empty
         else:
             rowcols = itertools.product(row_names, col_names)
-            name_dicts = [{row: r, col: c} for r, c in rowcols]
+            name_dict_list = [{row: r, col: c} for r, c in rowcols]
 
-        name_dicts = np.array(name_dicts).reshape(nrow, ncol)
+        name_dicts = np.array(name_dict_list).reshape(nrow, ncol)
 
         # Set up the class attributes
         # ---------------------------
@@ -222,14 +270,16 @@ class FacetGrid:
         self._finalized = False
 
     @property
-    def _left_axes(self):
+    def _left_axes(self) -> np.ndarray:
         return self.axes[:, 0]
 
     @property
-    def _bottom_axes(self):
+    def _bottom_axes(self) -> np.ndarray:
         return self.axes[-1, :]
 
-    def map_dataarray(self, func, x, y, **kwargs):
+    def map_dataarray(
+        self, func: Callable, x: Hashable | None, y: Hashable | None, **kwargs: Any
+    ) -> FacetGrid:
         """
         Apply a plotting function to a 2d facet's subset of the data.
 
@@ -296,9 +346,170 @@ class FacetGrid:
 
         return self
 
+    def map_plot1d(
+        self, func: Callable, x: Hashable, y: Hashable, **kwargs: Any
+    ) -> Self:
+        """
+        Apply a plotting function to a 1d facet's subset of the data.
+
+        This is more convenient and less general than ``FacetGrid.map``
+
+        Parameters
+        ----------
+        func :
+            A plotting function with the same signature as a 1d xarray
+            plotting method such as `xarray.plot.scatter`
+        x, y :
+            Names of the coordinates to plot on x, y axes
+        **kwargs
+            additional keyword arguments to func
+
+        Returns
+        -------
+        self : FacetGrid object
+
+        """
+        # Copy data to allow converting categoricals to integers and storing
+        # them in self.data. It is not possible to copy in the init
+        # unfortunately as there are tests that relies on self.data being
+        # mutable (test_names_appear_somewhere()). Maybe something to deprecate
+        # not sure how much that is used outside these tests.
+        self.data = self.data.copy()
+
+        if kwargs.get("cbar_ax", None) is not None:
+            raise ValueError("cbar_ax not supported by FacetGrid.")
+
+        # Handle hues:
+        hue = kwargs.get("hue", None)
+        hueplt = self.data[hue] if hue else self.data
+        hueplt_norm = _Normalize(hueplt)
+        self._hue_var = hueplt
+        cbar_kwargs = kwargs.pop("cbar_kwargs", {})
+        if not hueplt_norm.data_is_numeric:
+            # TODO: Ticks seems a little too hardcoded, since it will always
+            # show all the values. But maybe it's ok, since plotting hundreds
+            # of categorical data isn't that meaningful anyway.
+            cbar_kwargs.update(format=hueplt_norm.format, ticks=hueplt_norm.ticks)
+            kwargs.update(levels=hueplt_norm.levels)
+        if "label" not in cbar_kwargs:
+            cbar_kwargs["label"] = label_from_attrs(hueplt_norm.data)
+        cmap_params, cbar_kwargs = _process_cmap_cbar_kwargs(
+            func, hueplt_norm.values.to_numpy(), cbar_kwargs=cbar_kwargs, **kwargs
+        )
+        self._cmap_extend = cmap_params.get("extend")
+
+        # Handle sizes:
+        _size_r = _MARKERSIZE_RANGE if func.__name__ == "scatter" else _LINEWIDTH_RANGE
+        for _size in ("markersize", "linewidth"):
+            size = kwargs.get(_size, None)
+
+            sizeplt = self.data[size] if size else None
+            sizeplt_norm = _Normalize(sizeplt, _size_r)
+            if size:
+                self.data[size] = sizeplt_norm.values
+                kwargs.update(**{_size: size})
+                break
+
+        # Add kwargs that are sent to the plotting function, # order is important ???
+        func_kwargs = {
+            k: v
+            for k, v in kwargs.items()
+            if k not in {"cmap", "colors", "cbar_kwargs", "levels"}
+        }
+        func_kwargs.update(cmap_params)
+        # Annotations will be handled later, skip those parts in the plotfunc:
+        func_kwargs["add_colorbar"] = False
+        func_kwargs["add_legend"] = False
+        func_kwargs["add_title"] = False
+
+        add_labels_ = np.zeros(self.axes.shape + (3,), dtype=bool)
+        if kwargs.get("z") is not None:
+            # 3d plots looks better with all labels. 3d plots can't sharex either so it
+            # is easy to get lost while rotating the plots:
+            add_labels_[:] = True
+        else:
+            # Subplots should have labels on the left and bottom edges only:
+            add_labels_[-1, :, 0] = True  # x
+            add_labels_[:, 0, 1] = True  # y
+            # add_labels_[:, :, 2] = True  # z
+
+        # Set up the lists of names for the row and column facet variables:
+        if self._single_group:
+            full = tuple(
+                {self._single_group: x}
+                for x in range(0, self.data[self._single_group].size)
+            )
+            empty = tuple(None for x in range(self._nrow * self._ncol - len(full)))
+            name_d = full + empty
+        else:
+            rowcols = itertools.product(
+                range(0, self.data[self._row_var].size),
+                range(0, self.data[self._col_var].size),
+            )
+            name_d = tuple({self._row_var: r, self._col_var: c} for r, c in rowcols)
+        name_dicts = np.array(name_d).reshape(self._nrow, self._ncol)
+
+        # Plot the data for each subplot:
+        for add_lbls, d, ax in zip(
+            add_labels_.reshape((self.axes.size, -1)), name_dicts.flat, self.axes.flat
+        ):
+            func_kwargs["add_labels"] = add_lbls
+            # None is the sentinel value
+            if d is not None:
+                subset = self.data.isel(d)
+                mappable = func(
+                    subset,
+                    x=x,
+                    y=y,
+                    ax=ax,
+                    **func_kwargs,
+                    _is_facetgrid=True,
+                )
+                self._mappables.append(mappable)
+
+        # Add titles and some touch ups:
+        self._finalize_grid()
+        self._set_lims()
+
+        add_colorbar, add_legend = _determine_guide(
+            hueplt_norm,
+            sizeplt_norm,
+            kwargs.get("add_colorbar", None),
+            kwargs.get("add_legend", None),
+            # kwargs.get("add_guide", None),
+            # kwargs.get("hue_style", None),
+        )
+
+        if add_legend:
+            use_legend_elements = False if func.__name__ == "hist" else True
+            if use_legend_elements:
+                self.add_legend(
+                    use_legend_elements=use_legend_elements,
+                    hueplt_norm=hueplt_norm if not add_colorbar else _Normalize(None),
+                    sizeplt_norm=sizeplt_norm,
+                    primitive=self._mappables,
+                    legend_ax=self.fig,
+                    plotfunc=func.__name__,
+                )
+            else:
+                self.add_legend(use_legend_elements=use_legend_elements)
+
+        if add_colorbar:
+            # Colorbar is after legend so it correctly fits the plot:
+            self.add_colorbar(**cbar_kwargs)
+
+        return self
+
     def map_dataarray_line(
-        self, func, x, y, hue, add_legend=True, _labels=None, **kwargs
-    ):
+        self,
+        func: Callable,
+        x: Hashable | None,
+        y: Hashable | None,
+        hue: Hashable | None,
+        add_legend: bool = True,
+        _labels=None,
+        **kwargs: Any,
+    ) -> FacetGrid:
         from .plot import _infer_line_data
 
         for d, ax in zip(self.name_dicts.flat, self.axes.flat):
@@ -324,18 +535,23 @@ class FacetGrid:
         ylabel = label_from_attrs(yplt)
 
         self._hue_var = hueplt
-        self._hue_label = huelabel
         self._finalize_grid(xlabel, ylabel)
 
         if add_legend and hueplt is not None and huelabel is not None:
-            self.add_legend()
+            self.add_legend(label=huelabel)
 
         return self
 
     def map_dataset(
-        self, func, x=None, y=None, hue=None, hue_style=None, add_guide=None, **kwargs
-    ):
-        from .dataset_plot import _infer_meta_data, _parse_size
+        self,
+        func: Callable,
+        x: Hashable | None = None,
+        y: Hashable | None = None,
+        hue: Hashable | None = None,
+        hue_style: HueStyleOptions = None,
+        add_guide: bool | None = None,
+        **kwargs: Any,
+    ) -> FacetGrid:
 
         kwargs["add_guide"] = False
 
@@ -376,19 +592,20 @@ class FacetGrid:
         self._finalize_grid(meta_data["xlabel"], meta_data["ylabel"])
 
         if hue:
-            self._hue_label = meta_data.pop("hue_label", None)
+            hue_label = meta_data.pop("hue_label", None)
+            self._hue_label = hue_label
             if meta_data["add_legend"]:
                 self._hue_var = meta_data["hue"]
-                self.add_legend()
+                self.add_legend(label=hue_label)
             elif meta_data["add_colorbar"]:
-                self.add_colorbar(label=self._hue_label, **cbar_kwargs)
+                self.add_colorbar(label=hue_label, **cbar_kwargs)
 
         if meta_data["add_quiverkey"]:
             self.add_quiverkey(kwargs["u"], kwargs["v"])
 
         return self
 
-    def _finalize_grid(self, *axlabels):
+    def _finalize_grid(self, *axlabels: Hashable) -> None:
         """Finalize the annotations and layout."""
         if not self._finalized:
             self.set_axis_labels(*axlabels)
@@ -401,7 +618,7 @@ class FacetGrid:
 
             self._finalized = True
 
-    def _adjust_fig_for_guide(self, guide):
+    def _adjust_fig_for_guide(self, guide) -> None:
         # Draw the plot to set the bounding boxes correctly
         renderer = self.fig.canvas.get_renderer()
         self.fig.draw(renderer)
@@ -409,14 +626,15 @@ class FacetGrid:
         # Calculate and set the new width of the figure so the legend fits
         guide_width = guide.get_window_extent(renderer).width / self.fig.dpi
         figure_width = self.fig.get_figwidth()
-        self.fig.set_figwidth(figure_width + guide_width)
+        total_width = figure_width + guide_width
+        self.fig.set_figwidth(total_width)
 
         # Draw the plot again to get the new transformations
         self.fig.draw(renderer)
 
         # Now calculate how much space we need on the right side
         guide_width = guide.get_window_extent(renderer).width / self.fig.dpi
-        space_needed = guide_width / (figure_width + guide_width) + 0.02
+        space_needed = guide_width / total_width + 0.02
         # margin = .01
         # _space_needed = margin + space_needed
         right = 1 - space_needed
@@ -424,17 +642,26 @@ class FacetGrid:
         # Place the subplot axes to give space for the legend
         self.fig.subplots_adjust(right=right)
 
-    def add_legend(self, **kwargs):
-        self.figlegend = self.fig.legend(
-            handles=self._mappables[-1],
-            labels=list(self._hue_var.to_numpy()),
-            title=self._hue_label,
-            loc="center right",
-            **kwargs,
-        )
+    def add_legend(
+        self,
+        *,
+        label: str | None = None,
+        use_legend_elements: bool = False,
+        **kwargs: Any,
+    ) -> None:
+        if use_legend_elements:
+            self.figlegend = _add_legend(**kwargs)
+        else:
+            self.figlegend = self.fig.legend(
+                handles=self._mappables[-1],
+                labels=list(self._hue_var.to_numpy()),
+                title=label if label is not None else label_from_attrs(self._hue_var),
+                loc=kwargs.pop("loc", "center right"),
+                **kwargs,
+            )
         self._adjust_fig_for_guide(self.figlegend)
 
-    def add_colorbar(self, **kwargs):
+    def add_colorbar(self, **kwargs: Any) -> None:
         """Draw a colorbar."""
         kwargs = kwargs.copy()
         if self._cmap_extend is not None:
@@ -447,9 +674,8 @@ class FacetGrid:
         self.cbar = self.fig.colorbar(
             self._mappables[-1], ax=list(self.axes.flat), **kwargs
         )
-        return self
 
-    def add_quiverkey(self, u, v, **kwargs):
+    def add_quiverkey(self, u: Hashable, v: Hashable, **kwargs: Any) -> None:
         kwargs = kwargs.copy()
 
         magnitude = _get_nice_quiver_magnitude(self.data[u], self.data[v])
@@ -467,51 +693,123 @@ class FacetGrid:
         # TODO: does not work because self.quiverkey.get_window_extent(renderer) = 0
         # https://github.com/matplotlib/matplotlib/issues/18530
         # self._adjust_fig_for_guide(self.quiverkey.text)
-        return self
 
-    def set_axis_labels(self, x_var=None, y_var=None):
+    def _get_largest_lims(self) -> dict[str, tuple[float, float]]:
+        """
+        Get largest limits in the facetgrid.
+
+        Returns
+        -------
+        lims_largest : dict[str, tuple[float, float]]
+            Dictionary with the largest limits along each axis.
+
+        Examples
+        --------
+        >>> ds = xr.tutorial.scatter_example_dataset(seed=42)
+        >>> fg = ds.plot.scatter("A", "B", hue="y", row="x", col="w")
+        >>> round(fg._get_largest_lims()["x"][0], 3)
+        -0.334
+        """
+        lims_largest: dict[str, tuple[float, float]] = dict(
+            x=(np.inf, -np.inf), y=(np.inf, -np.inf), z=(np.inf, -np.inf)
+        )
+        for axis in ("x", "y", "z"):
+            # Find the plot with the largest xlim values:
+            lower, upper = lims_largest[axis]
+            for ax in self.axes.flat:
+                get_lim: None | Callable[[], tuple[float, float]] = getattr(
+                    ax, f"get_{axis}lim", None
+                )
+                if get_lim:
+                    lower_new, upper_new = get_lim()
+                    lower, upper = (min(lower, lower_new), max(upper, upper_new))
+            lims_largest[axis] = (lower, upper)
+
+        return lims_largest
+
+    def _set_lims(
+        self,
+        x: tuple[float, float] | None = None,
+        y: tuple[float, float] | None = None,
+        z: tuple[float, float] | None = None,
+    ) -> None:
+        """
+        Set the same limits for all the subplots in the facetgrid.
+
+        Parameters
+        ----------
+        x : tuple[float, float] or None, optional
+            x axis limits.
+        y : tuple[float, float] or None, optional
+            y axis limits.
+        z : tuple[float, float] or None, optional
+            z axis limits.
+
+        Examples
+        --------
+        >>> ds = xr.tutorial.scatter_example_dataset(seed=42)
+        >>> fg = ds.plot.scatter("A", "B", hue="y", row="x", col="w")
+        >>> fg._set_lims(x=(-0.3, 0.3), y=(0, 2), z=(0, 4))
+        >>> fg.axes[0, 0].get_xlim(), fg.axes[0, 0].get_ylim()
+        ((-0.3, 0.3), (0.0, 2.0))
+        """
+        lims_largest = self._get_largest_lims()
+
+        # Set limits:
+        for ax in self.axes.flat:
+            for (axis, data_limit), parameter_limit in zip(
+                lims_largest.items(), (x, y, z)
+            ):
+                set_lim = getattr(ax, f"set_{axis}lim", None)
+                if set_lim:
+                    set_lim(data_limit if parameter_limit is None else parameter_limit)
+
+    def set_axis_labels(self, *axlabels: Hashable) -> None:
         """Set axis labels on the left column and bottom row of the grid."""
-        if x_var is not None:
-            if x_var in self.data.coords:
-                self._x_var = x_var
-                self.set_xlabels(label_from_attrs(self.data[x_var]))
-            else:
-                # x_var is a string
-                self.set_xlabels(x_var)
+        from ..core.dataarray import DataArray
 
-        if y_var is not None:
-            if y_var in self.data.coords:
-                self._y_var = y_var
-                self.set_ylabels(label_from_attrs(self.data[y_var]))
-            else:
-                self.set_ylabels(y_var)
-        return self
+        for var, axis in zip(axlabels, ["x", "y", "z"]):
+            if var is not None:
+                if isinstance(var, DataArray):
+                    getattr(self, f"set_{axis}labels")(label_from_attrs(var))
+                else:
+                    getattr(self, f"set_{axis}labels")(str(var))
 
-    def set_xlabels(self, label=None, **kwargs):
+    def _set_labels(
+        self, axis: str, axes: Iterable, label: str | None = None, **kwargs
+    ) -> None:
+        if label is None:
+            label = label_from_attrs(self.data[getattr(self, f"_{axis}_var")])
+        for ax in axes:
+            getattr(ax, f"set_{axis}label")(label, **kwargs)
+
+    def set_xlabels(self, label: None | str = None, **kwargs: Any) -> None:
         """Label the x axis on the bottom row of the grid."""
-        if label is None:
-            label = label_from_attrs(self.data[self._x_var])
-        for ax in self._bottom_axes:
-            ax.set_xlabel(label, **kwargs)
-        return self
+        self._set_labels("x", self._bottom_axes, label, **kwargs)
 
-    def set_ylabels(self, label=None, **kwargs):
+    def set_ylabels(self, label: None | str = None, **kwargs: Any) -> None:
         """Label the y axis on the left column of the grid."""
-        if label is None:
-            label = label_from_attrs(self.data[self._y_var])
-        for ax in self._left_axes:
-            ax.set_ylabel(label, **kwargs)
-        return self
+        self._set_labels("y", self._left_axes, label, **kwargs)
 
-    def set_titles(self, template="{coord} = {value}", maxchar=30, size=None, **kwargs):
+    def set_zlabels(self, label: None | str = None, **kwargs: Any) -> None:
+        """Label the z axis."""
+        self._set_labels("z", self._left_axes, label, **kwargs)
+
+    def set_titles(
+        self,
+        template: str = "{coord} = {value}",
+        maxchar: int = 30,
+        size=None,
+        **kwargs,
+    ) -> None:
         """
         Draw titles either above each facet or on the grid margins.
 
         Parameters
         ----------
-        template : string
+        template : str, default: "{coord} = {value}"
             Template for plot titles containing {coord} and {value}
-        maxchar : int
+        maxchar : int, default: 30
             Truncate titles at maxchar
         **kwargs : keyword args
             additional arguments to matplotlib.text
@@ -553,6 +851,7 @@ class FacetGrid:
                     )
                 else:
                     handle.set_text(title)
+                    handle.update(kwargs)
 
             # The column titles on the top row
             for index, (ax, col_name, handle) in enumerate(
@@ -563,10 +862,14 @@ class FacetGrid:
                     self.col_labels[index] = ax.set_title(title, size=size, **kwargs)
                 else:
                     handle.set_text(title)
+                    handle.update(kwargs)
 
-        return self
-
-    def set_ticks(self, max_xticks=_NTICKS, max_yticks=_NTICKS, fontsize=_FONTSIZE):
+    def set_ticks(
+        self,
+        max_xticks: int = _NTICKS,
+        max_yticks: int = _NTICKS,
+        fontsize: str | int = _FONTSIZE,
+    ) -> None:
         """
         Set and control tick behavior.
 
@@ -596,9 +899,7 @@ class FacetGrid:
             ):
                 tick.label1.set_fontsize(fontsize)
 
-        return self
-
-    def map(self, func, *args, **kwargs):
+    def map(self, func: Callable, *args: Any, **kwargs: Any) -> FacetGrid:
         """
         Apply a plotting function to each facet's subset of the data.
 
@@ -640,23 +941,23 @@ class FacetGrid:
 
 
 def _easy_facetgrid(
-    data,
-    plotfunc,
-    kind,
-    x=None,
-    y=None,
-    row=None,
-    col=None,
-    col_wrap=None,
-    sharex=True,
-    sharey=True,
-    aspect=None,
-    size=None,
-    subplot_kws=None,
-    ax=None,
-    figsize=None,
-    **kwargs,
-):
+    data: DataArray,
+    plotfunc: Callable,
+    kind: Literal["line", "dataarray", "dataset", "plot1d"],
+    x: Hashable | None = None,
+    y: Hashable | None = None,
+    row: Hashable | None = None,
+    col: Hashable | None = None,
+    col_wrap: int | None = None,
+    sharex: bool = True,
+    sharey: bool = True,
+    aspect: float | None = None,
+    size: float | None = None,
+    subplot_kws: dict[str, Any] | None = None,
+    ax: Axes | None = None,
+    figsize: Iterable[float] | None = None,
+    **kwargs: Any,
+) -> FacetGrid:
     """
     Convenience method to call xarray.plot.FacetGrid from 2d plotting methods
 
@@ -670,6 +971,10 @@ def _easy_facetgrid(
         size = 3
     elif figsize is not None:
         raise ValueError("cannot provide both `figsize` and `size` arguments")
+    if kwargs.get("z") is not None:
+        # 3d plots doesn't support sharex, sharey, reset to mpl defaults:
+        sharex = False
+        sharey = False
 
     g = FacetGrid(
         data=data,
@@ -690,5 +995,10 @@ def _easy_facetgrid(
     if kind == "dataarray":
         return g.map_dataarray(plotfunc, x, y, **kwargs)
 
+    if kind == "plot1d":
+        return g.map_plot1d(plotfunc, x, y, **kwargs)
+
     if kind == "dataset":
         return g.map_dataset(plotfunc, x, y, **kwargs)
+
+    raise ValueError(f"kind must be one of `line`, `dataarray`, `dataset`, got {kind}")
