@@ -5,7 +5,7 @@ import textwrap
 import warnings
 from datetime import datetime
 from inspect import getfullargspec
-from typing import Any, Iterable, Mapping
+from typing import TYPE_CHECKING, Any, Callable, Hashable, Iterable, Mapping, Sequence
 
 import numpy as np
 import pandas as pd
@@ -28,10 +28,22 @@ try:
 except ImportError:
     cftime = None
 
+
+if TYPE_CHECKING:
+    from matplotlib.axes import Axes
+
+    from ..core.dataarray import DataArray
+
+    try:
+        import matplotlib.pyplot as plt
+    except ImportError:
+        plt: Any = None  # type: ignore
+
 ROBUST_PERCENTILE = 2.0
 
 # copied from seaborn
 _MARKERSIZE_RANGE = np.array([18.0, 72.0])
+_LINEWIDTH_RANGE = np.array([1.5, 6.0])
 
 
 def import_matplotlib_pyplot():
@@ -306,7 +318,9 @@ def _determine_cmap_params(
     )
 
 
-def _infer_xy_labels_3d(darray, x, y, rgb):
+def _infer_xy_labels_3d(
+    darray: DataArray, x: Hashable | None, y: Hashable | None, rgb: Hashable | None
+) -> tuple[Hashable, Hashable]:
     """
     Determine x and y labels for showing RGB images.
 
@@ -360,10 +374,16 @@ def _infer_xy_labels_3d(darray, x, y, rgb):
     assert rgb is not None
 
     # Finally, we pick out the red slice and delegate to the 2D version:
-    return _infer_xy_labels(darray.isel(**{rgb: 0}), x, y)
+    return _infer_xy_labels(darray.isel({rgb: 0}), x, y)
 
 
-def _infer_xy_labels(darray, x, y, imshow=False, rgb=None):
+def _infer_xy_labels(
+    darray: DataArray,
+    x: Hashable | None,
+    y: Hashable | None,
+    imshow: bool = False,
+    rgb: Hashable | None = None,
+) -> tuple[Hashable, Hashable]:
     """
     Determine x and y labels. For use in _plot2d
 
@@ -396,7 +416,8 @@ def _infer_xy_labels(darray, x, y, imshow=False, rgb=None):
     return x, y
 
 
-def _assert_valid_xy(darray, xy, name):
+# TODO: Can by used to more than x or y, rename?
+def _assert_valid_xy(darray: DataArray, xy: Hashable | None, name: str) -> None:
     """
     make sure x and y passed to plotting functions are valid
     """
@@ -410,12 +431,20 @@ def _assert_valid_xy(darray, xy, name):
 
     valid_xy = (set(darray.dims) | set(darray.coords)) - multiindex_dims
 
-    if xy not in valid_xy:
-        valid_xy_str = "', '".join(sorted(valid_xy))
-        raise ValueError(f"{name} must be one of None, '{valid_xy_str}'")
+    if (xy is not None) and (xy not in valid_xy):
+        valid_xy_str = "', '".join(sorted(tuple(str(v) for v in valid_xy)))
+        raise ValueError(
+            f"{name} must be one of None, '{valid_xy_str}'. Received '{xy}' instead."
+        )
 
 
-def get_axis(figsize=None, size=None, aspect=None, ax=None, **kwargs):
+def get_axis(
+    figsize: Iterable[float] | None = None,
+    size: float | None = None,
+    aspect: float | None = None,
+    ax: Axes | None = None,
+    **subplot_kws: Any,
+) -> Axes:
     try:
         import matplotlib as mpl
         import matplotlib.pyplot as plt
@@ -427,28 +456,32 @@ def get_axis(figsize=None, size=None, aspect=None, ax=None, **kwargs):
             raise ValueError("cannot provide both `figsize` and `ax` arguments")
         if size is not None:
             raise ValueError("cannot provide both `figsize` and `size` arguments")
-        _, ax = plt.subplots(figsize=figsize)
-    elif size is not None:
+        _, ax = plt.subplots(figsize=figsize, subplot_kw=subplot_kws)
+        return ax
+
+    if size is not None:
         if ax is not None:
             raise ValueError("cannot provide both `size` and `ax` arguments")
         if aspect is None:
             width, height = mpl.rcParams["figure.figsize"]
             aspect = width / height
         figsize = (size * aspect, size)
-        _, ax = plt.subplots(figsize=figsize)
-    elif aspect is not None:
+        _, ax = plt.subplots(figsize=figsize, subplot_kw=subplot_kws)
+        return ax
+
+    if aspect is not None:
         raise ValueError("cannot provide `aspect` argument without `size`")
 
-    if kwargs and ax is not None:
+    if subplot_kws and ax is not None:
         raise ValueError("cannot use subplot_kws with existing ax")
 
     if ax is None:
-        ax = _maybe_gca(**kwargs)
+        ax = _maybe_gca(**subplot_kws)
 
     return ax
 
 
-def _maybe_gca(**kwargs):
+def _maybe_gca(**subplot_kws: Any) -> Axes:
 
     import matplotlib.pyplot as plt
 
@@ -460,10 +493,10 @@ def _maybe_gca(**kwargs):
         # can not pass kwargs to active axes
         return plt.gca()
 
-    return plt.axes(**kwargs)
+    return plt.axes(**subplot_kws)
 
 
-def _get_units_from_attrs(da):
+def _get_units_from_attrs(da) -> str:
     """Extracts and formats the unit/units from a attributes."""
     pint_array_type = DuckArrayModule("pint").type
     units = " [{}]"
@@ -478,16 +511,16 @@ def _get_units_from_attrs(da):
     return units
 
 
-def label_from_attrs(da, extra=""):
+def label_from_attrs(da, extra: str = "") -> str:
     """Makes informative labels if variable metadata (attrs) follows
     CF conventions."""
-
+    name: str = "{}"
     if da.attrs.get("long_name"):
-        name = da.attrs["long_name"]
+        name = name.format(da.attrs["long_name"])
     elif da.attrs.get("standard_name"):
-        name = da.attrs["standard_name"]
+        name = name.format(da.attrs["standard_name"])
     elif da.name is not None:
-        name = da.name
+        name = name.format(da.name)
     else:
         name = ""
 
@@ -502,7 +535,7 @@ def label_from_attrs(da, extra=""):
         return "\n".join(textwrap.wrap(name + extra + units, 30))
 
 
-def _interval_to_mid_points(array):
+def _interval_to_mid_points(array: Iterable[pd.Interval]) -> np.ndarray:
     """
     Helper function which returns an array
     with the Intervals' mid points.
@@ -511,7 +544,7 @@ def _interval_to_mid_points(array):
     return np.array([x.mid for x in array])
 
 
-def _interval_to_bound_points(array):
+def _interval_to_bound_points(array: Sequence[pd.Interval]) -> np.ndarray:
     """
     Helper function which returns an array
     with the Intervals' boundaries.
@@ -523,7 +556,9 @@ def _interval_to_bound_points(array):
     return array_boundaries
 
 
-def _interval_to_double_bound_points(xarray, yarray):
+def _interval_to_double_bound_points(
+    xarray: Iterable[pd.Interval], yarray: Iterable
+) -> tuple[np.ndarray, np.ndarray]:
     """
     Helper function to deal with a xarray consisting of pd.Intervals. Each
     interval is replaced with both boundaries. I.e. the length of xarray
@@ -533,13 +568,15 @@ def _interval_to_double_bound_points(xarray, yarray):
     xarray1 = np.array([x.left for x in xarray])
     xarray2 = np.array([x.right for x in xarray])
 
-    xarray = list(itertools.chain.from_iterable(zip(xarray1, xarray2)))
-    yarray = list(itertools.chain.from_iterable(zip(yarray, yarray)))
+    xarray_out = np.array(list(itertools.chain.from_iterable(zip(xarray1, xarray2))))
+    yarray_out = np.array(list(itertools.chain.from_iterable(zip(yarray, yarray))))
 
-    return xarray, yarray
+    return xarray_out, yarray_out
 
 
-def _resolve_intervals_1dplot(xval, yval, kwargs):
+def _resolve_intervals_1dplot(
+    xval: np.ndarray, yval: np.ndarray, kwargs: dict
+) -> tuple[np.ndarray, np.ndarray, str, str, dict]:
     """
     Helper function to replace the values of x and/or y coordinate arrays
     containing pd.Interval with their mid-points or - for step plots - double
@@ -552,13 +589,16 @@ def _resolve_intervals_1dplot(xval, yval, kwargs):
     if kwargs.get("drawstyle", "").startswith("steps-"):
 
         remove_drawstyle = False
+
         # Convert intervals to double points
-        if _valid_other_type(np.array([xval, yval]), [pd.Interval]):
+        x_is_interval = _valid_other_type(xval, [pd.Interval])
+        y_is_interval = _valid_other_type(yval, [pd.Interval])
+        if x_is_interval and y_is_interval:
             raise TypeError("Can't step plot intervals against intervals.")
-        if _valid_other_type(xval, [pd.Interval]):
+        elif x_is_interval:
             xval, yval = _interval_to_double_bound_points(xval, yval)
             remove_drawstyle = True
-        if _valid_other_type(yval, [pd.Interval]):
+        elif y_is_interval:
             yval, xval = _interval_to_double_bound_points(yval, xval)
             remove_drawstyle = True
 
@@ -1008,7 +1048,10 @@ def legend_elements(
             return self.cmap(self.norm(value)), _size
 
     elif prop == "sizes":
-        arr = self.get_sizes()
+        if isinstance(self, mpl.collections.LineCollection):
+            arr = self.get_linewidths()
+        else:
+            arr = self.get_sizes()
         _color = kwargs.pop("color", "k")
 
         def _get_color_and_size(value):
@@ -1097,22 +1140,29 @@ def legend_elements(
 
     for val, lab in zip(values, label_values):
         color, size = _get_color_and_size(val)
-        h = mlines.Line2D(
-            [0], [0], ls="", color=color, ms=size, marker=self.get_paths()[0], **kw
-        )
+
+        if isinstance(self, mpl.collections.PathCollection):
+            kw.update(linestyle="", marker=self.get_paths()[0], markersize=size)
+        elif isinstance(self, mpl.collections.LineCollection):
+            kw.update(linestyle=self.get_linestyle()[0], linewidth=size)
+
+        h = mlines.Line2D([0], [0], color=color, **kw)
+
         handles.append(h)
         labels.append(fmt(lab))
 
     return handles, labels
 
 
-def _legend_add_subtitle(handles, labels, text, func):
+def _legend_add_subtitle(handles, labels, text):
     """Add a subtitle to legend handles."""
+    plt = import_matplotlib_pyplot()
+
     if text and len(handles) > 1:
         # Create a blank handle that's not visible, the
         # invisibillity will be used to discern which are subtitles
         # or not:
-        blank_handle = func([], [], label=text)
+        blank_handle = plt.Line2D([], [], label=text)
         blank_handle.set_visible(False)
 
         # Subtitles are shown first:
@@ -1129,8 +1179,13 @@ def _adjust_legend_subtitles(legend):
     # Legend title not in rcParams until 3.0
     font_size = plt.rcParams.get("legend.title_fontsize", None)
     hpackers = legend.findobj(plt.matplotlib.offsetbox.VPacker)[0].get_children()
+    hpackers = [v for v in hpackers if isinstance(v, plt.matplotlib.offsetbox.HPacker)]
     for hpack in hpackers:
-        draw_area, text_area = hpack.get_children()
+        areas = hpack.get_children()
+        if len(areas) < 2:
+            continue
+        draw_area, text_area = areas
+
         handles = draw_area.get_children()
 
         # Assume that all artists that are not visible are
@@ -1148,16 +1203,16 @@ def _adjust_legend_subtitles(legend):
 
 def _infer_meta_data(ds, x, y, hue, hue_style, add_guide, funcname):
     dvars = set(ds.variables.keys())
-    error_msg = " must be one of ({:s})".format(", ".join(dvars))
+    error_msg = f" must be one of ({', '.join(sorted(tuple(str(v) for v in dvars)))})"
 
     if x not in dvars:
-        raise ValueError("x" + error_msg)
+        raise ValueError(f"Expected 'x' {error_msg}. Received {x} instead.")
 
     if y not in dvars:
-        raise ValueError("y" + error_msg)
+        raise ValueError(f"Expected 'y' {error_msg}. Received {y} instead.")
 
     if hue is not None and hue not in dvars:
-        raise ValueError("hue" + error_msg)
+        raise ValueError(f"Expected 'hue' {error_msg}. Received {hue} instead.")
 
     if hue:
         hue_is_numeric = _is_numeric(ds[hue].values)
@@ -1268,3 +1323,330 @@ def _parse_size(data, norm):
     sizes = dict(zip(levels, widths))
 
     return pd.Series(sizes)
+
+
+class _Normalize(Sequence):
+    """
+    Normalize numerical or categorical values to numerical values.
+
+    The class includes helper methods that simplifies transforming to
+    and from normalized values.
+
+    Parameters
+    ----------
+    data : DataArray
+        DataArray to normalize.
+    width : Sequence of two numbers, optional
+        Normalize the data to theses min and max values.
+        The default is None.
+    """
+
+    __slots__ = (
+        "_data",
+        "_data_is_numeric",
+        "_width",
+        "_unique",
+        "_unique_index",
+        "_unique_inverse",
+        "plt",
+    )
+
+    def __init__(self, data, width=None, _is_facetgrid=False):
+        self._data = data
+        self._width = width if not _is_facetgrid else None
+        self.plt = import_matplotlib_pyplot()
+
+        pint_array_type = DuckArrayModule("pint").type
+        to_unique = data.to_numpy() if isinstance(self._type, pint_array_type) else data
+        unique, unique_inverse = np.unique(to_unique, return_inverse=True)
+        self._unique = unique
+        self._unique_index = np.arange(0, unique.size)
+        if data is not None:
+            self._unique_inverse = data.copy(data=unique_inverse.reshape(data.shape))
+            self._data_is_numeric = _is_numeric(data)
+        else:
+            self._unique_inverse = unique_inverse
+            self._data_is_numeric = False
+
+    def __repr__(self) -> str:
+        with np.printoptions(precision=4, suppress=True, threshold=5):
+            return (
+                f"<_Normalize(data, width={self._width})>\n"
+                f"{self._unique} -> {self.values_unique}"
+            )
+
+    def __len__(self) -> int:
+        return len(self._unique)
+
+    def __getitem__(self, key):
+        return self._unique[key]
+
+    @property
+    def _type(self):
+        data = self.data
+        return data.data if data is not None else data
+
+    @property
+    def data(self):
+        return self._data
+
+    @property
+    def data_is_numeric(self) -> bool:
+        """
+        Check if data is numeric.
+
+        Examples
+        --------
+        >>> a = xr.DataArray(["b", "a", "a", "b", "c"])
+        >>> _Normalize(a).data_is_numeric
+        False
+        """
+        return self._data_is_numeric
+
+    def _calc_widths(self, y):
+        if self._width is None or y is None:
+            return y
+
+        x0, x1 = self._width
+
+        k = (y - np.min(y)) / (np.max(y) - np.min(y))
+        widths = x0 + k * (x1 - x0)
+
+        return widths
+
+    def _indexes_centered(self, x) -> None | Any:
+        """
+        Offset indexes to make sure being in the center of self.levels.
+        ["a", "b", "c"] -> [1, 3, 5]
+        """
+        if self.data is None:
+            return None
+        else:
+            return x * 2 + 1
+
+    @property
+    def values(self):
+        """
+        Return a normalized number array for the unique levels.
+
+        Examples
+        --------
+        >>> a = xr.DataArray(["b", "a", "a", "b", "c"])
+        >>> _Normalize(a).values
+        <xarray.DataArray (dim_0: 5)>
+        array([3, 1, 1, 3, 5])
+        Dimensions without coordinates: dim_0
+
+        >>> _Normalize(a, width=[18, 72]).values
+        <xarray.DataArray (dim_0: 5)>
+        array([45., 18., 18., 45., 72.])
+        Dimensions without coordinates: dim_0
+
+        >>> a = xr.DataArray([0.5, 0, 0, 0.5, 2, 3])
+        >>> _Normalize(a).values
+        <xarray.DataArray (dim_0: 6)>
+        array([0.5, 0. , 0. , 0.5, 2. , 3. ])
+        Dimensions without coordinates: dim_0
+
+        >>> _Normalize(a, width=[18, 72]).values
+        <xarray.DataArray (dim_0: 6)>
+        array([27., 18., 18., 27., 54., 72.])
+        Dimensions without coordinates: dim_0
+        """
+        return self._calc_widths(
+            self.data
+            if self.data_is_numeric
+            else self._indexes_centered(self._unique_inverse)
+        )
+
+    def _integers(self):
+        """
+        Return integers.
+        ["a", "b", "c"] -> [1, 3, 5]
+        """
+        return self._indexes_centered(self._unique_index)
+
+    @property
+    def values_unique(self) -> np.ndarray:
+        """
+        Return unique values.
+
+        Examples
+        --------
+        >>> a = xr.DataArray(["b", "a", "a", "b", "c"])
+        >>> _Normalize(a).values_unique
+        array([1, 3, 5])
+        >>> a = xr.DataArray([2, 1, 1, 2, 3])
+        >>> _Normalize(a).values_unique
+        array([1, 2, 3])
+        >>> _Normalize(a, width=[18, 72]).values_unique
+        array([18., 45., 72.])
+        """
+        return (
+            self._integers()
+            if not self.data_is_numeric
+            else self._calc_widths(self._unique)
+        )
+
+    @property
+    def ticks(self) -> None | np.ndarray:
+        """
+        Return ticks for plt.colorbar if the data is not numeric.
+
+        Examples
+        --------
+        >>> a = xr.DataArray(["b", "a", "a", "b", "c"])
+        >>> _Normalize(a).ticks
+        array([1, 3, 5])
+        """
+        return self._integers() if not self.data_is_numeric else None
+
+    @property
+    def levels(self) -> np.ndarray:
+        """
+        Return discrete levels that will evenly bound self.values.
+        ["a", "b", "c"] -> [0, 2, 4, 6]
+
+        Examples
+        --------
+        >>> a = xr.DataArray(["b", "a", "a", "b", "c"])
+        >>> _Normalize(a).levels
+        array([0, 2, 4, 6])
+        """
+        return np.append(self._unique_index, np.max(self._unique_index) + 1) * 2
+
+    @property
+    def _lookup(self) -> pd.Series:
+        return pd.Series(dict(zip(self.values_unique, self._unique)))
+
+    def _lookup_arr(self, x) -> np.ndarray:
+        # Use reindex to be less sensitive to float errors. reindex only
+        # works with sorted index.
+        # Return as numpy array since legend_elements
+        # seems to require that:
+        return self._lookup.sort_index().reindex(x, method="nearest").to_numpy()
+
+    @property
+    def format(self) -> plt.FuncFormatter:
+        """
+        Return a FuncFormatter that maps self.values elements back to
+        the original value as a string. Useful with plt.colorbar.
+
+        Examples
+        --------
+        >>> a = xr.DataArray([0.5, 0, 0, 0.5, 2, 3])
+        >>> aa = _Normalize(a, width=[0, 1])
+        >>> aa._lookup
+        0.000000    0.0
+        0.166667    0.5
+        0.666667    2.0
+        1.000000    3.0
+        dtype: float64
+        >>> aa.format(1)
+        '3.0'
+        """
+
+        def _func(x: Any, pos: None | Any = None):
+            return f"{self._lookup_arr([x])[0]}"
+
+        return self.plt.FuncFormatter(_func)
+
+    @property
+    def func(self) -> Callable[[Any, None | Any], Any]:
+        """
+        Return a lambda function that maps self.values elements back to
+        the original value as a numpy array. Useful with ax.legend_elements.
+
+        Examples
+        --------
+        >>> a = xr.DataArray([0.5, 0, 0, 0.5, 2, 3])
+        >>> aa = _Normalize(a, width=[0, 1])
+        >>> aa._lookup
+        0.000000    0.0
+        0.166667    0.5
+        0.666667    2.0
+        1.000000    3.0
+        dtype: float64
+        >>> aa.func([0.16, 1])
+        array([0.5, 3. ])
+        """
+
+        def _func(x: Any, pos: None | Any = None):
+            return self._lookup_arr(x)
+
+        return _func
+
+
+def _determine_guide(
+    hueplt_norm: _Normalize,
+    sizeplt_norm: _Normalize,
+    add_colorbar: None | bool = None,
+    add_legend: None | bool = None,
+    plotfunc_name: str = None,
+) -> tuple[bool, bool]:
+    if plotfunc_name == "hist":
+        return False, False
+
+    if (add_colorbar) and hueplt_norm.data is None:
+        raise KeyError("Cannot create a colorbar when hue is None.")
+    if add_colorbar is None:
+        if hueplt_norm.data is not None:
+            add_colorbar = True
+        else:
+            add_colorbar = False
+
+    if (add_legend) and hueplt_norm.data is None and sizeplt_norm.data is None:
+        raise KeyError("Cannot create a legend when hue and markersize is None.")
+    if add_legend is None:
+        if (
+            not add_colorbar
+            and (hueplt_norm.data is not None and hueplt_norm.data_is_numeric is False)
+            or sizeplt_norm.data is not None
+        ):
+            add_legend = True
+        else:
+            add_legend = False
+
+    return add_colorbar, add_legend
+
+
+def _add_legend(
+    hueplt_norm: _Normalize,
+    sizeplt_norm: _Normalize,
+    primitive,
+    legend_ax,
+    plotfunc: str,
+):
+
+    primitive = primitive if isinstance(primitive, list) else [primitive]
+
+    handles, labels = [], []
+    for huesizeplt, prop in [
+        (hueplt_norm, "colors"),
+        (sizeplt_norm, "sizes"),
+    ]:
+        if huesizeplt.data is not None:
+            # Get legend handles and labels that displays the
+            # values correctly. Order might be different because
+            # legend_elements uses np.unique instead of pd.unique,
+            # FacetGrid.add_legend might have troubles with this:
+            hdl, lbl = [], []
+            for p in primitive:
+                hdl_, lbl_ = legend_elements(p, prop, num="auto", func=huesizeplt.func)
+                hdl += hdl_
+                lbl += lbl_
+
+            # Only save unique values:
+            u, ind = np.unique(lbl, return_index=True)
+            ind = np.argsort(ind)
+            lbl = u[ind].tolist()
+            hdl = np.array(hdl)[ind].tolist()
+
+            # Add a subtitle:
+            hdl, lbl = _legend_add_subtitle(hdl, lbl, label_from_attrs(huesizeplt.data))
+            handles += hdl
+            labels += lbl
+    legend = legend_ax.legend(handles, labels, framealpha=0.5)
+    _adjust_legend_subtitles(legend)
+
+    return legend
